!!$=head1 NAME
!!$
!!$Cross - Calculate the correlations needed for designing shaping filter
!!$
!!$=head1 SYNOPSIS
!!$
!!$PZ < in.H design_wind_top= design_wind_bottom= corr_length= pcorr= zcorr= pzcross= displayz= displayp= displayzp= > /dev/null
!!$
!!$=head1 DESCRIPTION
!!$
!!$ Compute the autocorrelations needed for designing the shaping filter for the PZ summation using the method of PZ
!!$ calibration by applying the equation of motion to critically refracted waves" Melbo et al. (2002)
!!$
!!$=head1 PARAMETERS
!!$
!!$=over 3
!!$
!!$=item from history 
!!$
!!$=item from par
!!$
!!$B<filename> I<design_wind_top> - Table containing the time, sx, sy, gx, gy for the top pickings of the windows
!!$
!!$B<filename> I<design_wind_bottom> - Table containing the time, sx, sy, gx, gy for the bottom pickings of the windows
!!$
!!$B<real> I<corr_length> - Length of the autocorrelations and crosscorrelation files (one side seconds)
!!$
!!$B<filename> I<pcorr> - Name of the file containing the Autocorrelation of the Pressure component
!!$
!!$B<filename> I<zcorr> - Name of the file containing the Autocorrelation of the vertical component
!!$
!!$B<filename> I<pzcross> - Name of the file containing the Crosscorrelation of the Pressure with the vertical component
!!$
!!$B<filename> I<displayz> - Name of the file containing the autocorrelation for the paper figure
!!$
!!$B<filename> I<displayp> - Name of the file containing the autocorrelation for the paper figure
!!$
!!$B<filename> I<displayzp> - Name of the file containing the crosscorrelation (zp) for the papaer figure
!!$
!!$
!!$=back
!!$
!!$=head1 EXAMPLES
!!$
!!$=head1 MODULES
!!$
!!$=head1 SEE ALSO
!!$
!!$=cut
!%
program Corr

  use sep
  use pick_obj
  use ieee_arithmetic
  use correlation



  implicit none
  type(pick) , pointer                   :: picking(:)
  integer                                :: n1, n2
  integer                                :: iheader, gxid, gyid, sxid, syid, lagaid, headlen
  integer, dimension(:),allocatable      :: header
  real                                   :: d1, o1
  real(kind=8) 			         :: t(2)
  character*256  		    	 :: topfile, bottomfile, pcorrfile, zcorrfile, pzcrossfile
  integer				 :: TOP, BOTTOM
  logical	     		    	 :: insd(2)
  real, dimension(:,:), allocatable      :: component_traces
  real    				 :: corr_length, o1_corr
  integer				 :: ncorr !number of samples per trace of the correlation files
  integer				 :: top_ix, bottom_ix
  real, dimension(:,:), allocatable	 :: corr_buffer ! buffer for the autocorrelations of P and Z, and crosscorrelation of PZ
  integer				 :: PAUTO,ZAUTO,PZCROSS
  integer				 :: PCOMP,XCOMP,YCOMP,ZCOMP
  integer				 :: ptrace_ix,ztrace_ix
  logical				 :: first_gather, last_gather
  integer				 :: sreed, rc, itrace
  integer				 :: srite, wc, num_rec
  integer				 :: gx_check, gy_check
  integer				 :: hc, hwc, sep_put_key, sep_put_number_keys, sep_put_val_headers, sep_put_header_axis_par
  integer				 :: GXHD,GYHD,SLPHD
  real					 :: tfirst(2),offset,sxfirst,syfirst,slope(2), hor_slow
  logical				 :: first_slope, slope_done
  integer				 :: hdrvals(3),dsi


  parameter(TOP=1,BOTTOM=2)
  parameter(PAUTO=1,ZAUTO=2,PZCROSS=3) 
  parameter(PCOMP=1,XCOMP=3,YCOMP=4,ZCOMP=2)
  parameter(GXHD=1,GYHD=2,SLPHD=3)

  call sep_init (SOURCE)
  call init_3d ()
  call from_either ("n1", n1)
  call from_either ("d1",d1)
  call from_either ("o1",o1)
  call from_either ("n2", n2)
  call from_par ("design_wind_top",topfile)
  call from_par ("design_wind_bottom", bottomfile)
  call from_par ("corr_length",corr_length)
  call from_par ("pcorr",pcorrfile)
  call from_par ("zcorr",zcorrfile)
  call from_par ("pzcross",pzcrossfile)
  




  allocate(picking(2))
  allocate(component_traces(n1,4))

  !Initialize internal variables
  t(:) = 0
  insd = .false.
  first_gather = .true.
  last_gather = .false.
  num_rec = 1
  first_slope = .false.
  slope_done = .false.

  ncorr = nint(corr_length/d1)*2+1 
  allocate(corr_buffer(ncorr,6))!Contains also the two writing buffer for the display of the correlations
  corr_buffer(:,:) = 0
  dsi = 0

  o1_corr = -corr_length
  !Writing information on header
  call to_history("n1",ncorr,"pcorr")
  call to_history("n1",ncorr,"zcorr")
  call to_history("n1",ncorr,"pzcross")
  call to_history("n1",ncorr,"displayz")
  call to_history("n1",ncorr,"displayp")
  call to_history("n1",ncorr,"displayzp")
  call to_history("d1",d1,"pcorr")
  call to_history("d1",d1,"zcorr")
  call to_history("d1",d1,"pzcross")
  call to_history("d1",d1,"displayz")
  call to_history("d1",d1,"displayp")
  call to_history("d1",d1,"displayzp")
  call to_history("o1",o1_corr,"pcorr")
  call to_history("o1",o1_corr,"zcorr")
  call to_history("o1",o1_corr,"pzcross")
  call to_history("o1",o1_corr,"displayz")
  call to_history("o1",o1_corr,"displayp")
  call to_history("o1",o1_corr,"displayzp")

  call to_history("d2",1.,"pcorr")
  call to_history("d2",1.,"zcorr")
  call to_history("d2",1.,"pzcross")
  call to_history("o2",1.,"pcorr")
  call to_history("o2",1.,"zcorr")
  call to_history("o2",1.,"pzcross")
  call to_history("o2",1.,"displayz")
  call to_history("o2",1.,"displayp")
  call to_history("o2",1.,"displayzp")
   !Create header entries
  hc = sep_put_number_keys("pcorr",3)
  hc = sep_put_number_keys("zcorr",3)
  hc = sep_put_number_keys("pzcross",3)
  hc = sep_put_key("pcorr","gx","scalar_int",'xdr_int',GXHD)
  hc = sep_put_key("zcorr","gx","scalar_int",'xdr_int',GXHD)
  hc = sep_put_key("pzcross","gx","scalar_int",'xdr_int',GXHD)
  hc = sep_put_key("pcorr","gy","scalar_int",'xdr_int',GYHD)
  hc = sep_put_key("zcorr","gy","scalar_int",'xdr_int',GYHD)
  hc = sep_put_key("pzcross","gy","scalar_int",'xdr_int',GYHD)
  hc = sep_put_key("pcorr","apparent_velocity","scalar_int",'xdr_int',SLPHD)
  hc = sep_put_key("zcorr","apparent_velocity","scalar_int",'xdr_int',SLPHD)
  hc = sep_put_key("pzcross","apparent_velocity","scalar_int",'xdr_int',SLPHD)


  !Initialize the value for the receiver change checking
  picking(TOP)%gxchk = -huge(picking(TOP)%gxchk)
  picking(TOP)%gychk = -huge(picking(TOP)%gychk)
  picking(BOTTOM)%gxchk = -huge(picking(BOTTOM)%gxchk)
  picking(BOTTOM)%gychk = -huge(picking(BOTTOM)%gychk)
  !Allocate and reading the picking tabs
  call picking(TOP)%decode_pick_file(topfile) !picking(1) = top pickings
  call picking(BOTTOM)%decode_pick_file(bottomfile) !picking(2) = bottom pickings
  !Create unit vector of the picked receivers
  call picking(TOP)%vector
  call picking(BOTTOM)%vector
  !call picking(1)%writing_values
  !call picking(2)%writing_values
  

  call sep_get_key_index("in", 'gx', gxid)
  call sep_get_key_index("in", 'gy', gyid)
  call sep_get_key_index("in", 'sx', sxid)
  call sep_get_key_index("in", 'sy', syid)
  call sep_get_key_index("in", 'laga', lagaid) 
  call sep_get_number_keys("in", headlen)
  allocate(header(headlen))
!Initialize all the variables
  iheader=1 
  rc=1
  do while (rc > 0)
    !read for traces at each time
    rc=sreed("in" , component_traces(:,1) , n1*4)
    rc=sreed("in" , component_traces(:,2) , n1*4)
    rc=sreed("in" , component_traces(:,3) , n1*4)
    rc=sreed("in" , component_traces(:,4) , n1*4)
    if(rc < 1) exit!last_gather=.true.
    do itrace= 1,4
       call sep_get_val_headers("in", iheader, 1, header) 
       iheader= iheader+1
       if (header(lagaid) == PCOMP) ptrace_ix=itrace
       if (header(lagaid) == ZCOMP) ztrace_ix=itrace
    enddo
    insd(TOP)= picking(TOP)%window(header(gxid), header(gyid), header(sxid), header(syid), t(TOP))
    insd(BOTTOM)= picking(BOTTOM)%window(header(gxid), header(gyid), header(sxid), header(syid), t(BOTTOM)) 
    !write (0,*) 'Bytes read =',rc
    if ( .not. (insd(1) .and. insd(2)) ) cycle
    if (first_gather) then
       gx_check = header(gxid)
       gy_check = header(gyid)
       first_gather = .false.
    end if
    !Compute the slope of the window (i.e. horizontal slowness)
    if (first_slope .and. (.not. slope_done)) then
	offset = sqrt((real(header(sxid))-sxfirst)**2. + (real(header(syid))-syfirst)**2.) / 100. ! from cm to meter
	slope(TOP) = offset / (t(TOP) - tfirst(TOP)) 
	slope(BOTTOM) = offset / (t(BOTTOM) - tfirst(BOTTOM)) 
	hor_slow = (slope(TOP) + slope(BOTTOM) )/ 2.
	!hor_slow = 1./hor_slow
	slope_done = .true.
    endif
    if ( .not. first_slope) then
       	tfirst(TOP) = t(TOP)
	tfirst(BOTTOM) = t(BOTTOM)
	sxfirst = real(header(sxid))
	syfirst = real(header(syid))
	first_slope = .true.
    endif
    !Check if it is another receiver gather
    if ((gx_check .ne. header(gxid)) .or. (gy_check .ne. header(gyid)) ) then
       !Write the headers values
       hdrvals(1) = gx_check
       hdrvals(2) = gy_check
       hdrvals(3) = hor_slow
       hwc = sep_put_val_headers ("pcorr",num_rec,1,hdrvals)
       hwc = sep_put_val_headers ("zcorr",num_rec,1,hdrvals)
       hwc = sep_put_val_headers ("pzcross",num_rec,1,hdrvals)
       !Change the receiver gather position
       gx_check = header(gxid)
       gy_check = header(gyid)
       !Write the result for the gather and then flush the corr_buffer
       wc=srite("pcorr",corr_buffer(:, PAUTO),ncorr*4)
       wc=srite("zcorr",corr_buffer(:, ZAUTO),ncorr*4)
       wc=srite("pzcross",corr_buffer(:, PZCROSS),ncorr*4)
       write(0,*)"Written receiver number ", num_rec
       num_rec = num_rec + 1
       corr_buffer(:,:) = 0
       slope_done = .false.
       first_slope = .false.
       dsi=1
    endif
    top_ix = nint(t(TOP)/d1)
    bottom_ix = nint(t(BOTTOM)/d1)
    call correl(component_traces(:,ptrace_ix), component_traces(:,ptrace_ix), corr_buffer(:, PAUTO), top_ix, bottom_ix) !compute autoc of P
    call correl(component_traces(:,ztrace_ix), component_traces(:,ztrace_ix), corr_buffer(:, ZAUTO), top_ix, bottom_ix) !compute autoc of Z
    call correl(component_traces(:,ptrace_ix), component_traces(:,ztrace_ix), corr_buffer(:, PZCROSS), top_ix, bottom_ix) !compute cross of PZ
    !Code portion to show pzcross and zauto for each shot for a receiver
	if (dsi.eq.0) then
		corr_buffer(:,4) = corr_buffer(:,ZAUTO)
		corr_buffer(:,5) = corr_buffer(:,PAUTO)
		corr_buffer(:,6) = corr_buffer(:,PZCROSS)
		wc=srite("displayz",corr_buffer(:,ZAUTO),ncorr*4)
		wc=srite("displayp",corr_buffer(:,PAUTO),ncorr*4)
		wc=srite("displayzp",corr_buffer(ncorr:1:-1, PZCROSS),ncorr*4)
		dsi = dsi+1
	else
		wc=srite("displayz",(corr_buffer(:, ZAUTO)-corr_buffer(:,4)),ncorr*4)
		wc=srite("displayp",(corr_buffer(:, PAUTO)-corr_buffer(:,5)),ncorr*4)
		wc=srite("displayzp",(corr_buffer(ncorr:1:-1,PZCROSS)-corr_buffer(ncorr:1:-1,6)),ncorr*4)
		corr_buffer(:,4) = corr_buffer(:,ZAUTO)
		corr_buffer(:,5) = corr_buffer(:,PAUTO)
		corr_buffer(:,6) = corr_buffer(:,PZCROSS)
		dsi = dsi+1
	endif
  enddo
  
  !Write the last gather correlations and the number of receiver gathers
  wc=srite("pcorr",corr_buffer(:, PAUTO),ncorr*4)
  wc=srite("zcorr",corr_buffer(:, ZAUTO),ncorr*4)
  wc=srite("pzcross",corr_buffer(:, PZCROSS),ncorr*4)
  write(0,*)"Written last receiver"
  call to_history("n2",num_rec,"pcorr")
  call to_history("n2",num_rec,"zcorr")
  call to_history("n2",num_rec,"pzcross")
  call to_history("d2",1.,"pcorr")
  call to_history("d2",1.,"zcorr")
  call to_history("d2",1.,"pzcross")
  call to_history("n2",dsi,"displayz")
  call to_history("n2",dsi,"displayp")
  call to_history("n2",dsi,"displayzp")
  hwc = sep_put_header_axis_par("pcorr",2,num_rec,1.,1.,"receiver number")
  hwc = sep_put_header_axis_par("zcorr",2,num_rec,1.,1.,"receiver number")
  hwc = sep_put_header_axis_par("pzcross",2,num_rec,1.,1.,"receiver number")
  !Write the headers values for the last receiver
       hdrvals(1) = gx_check
       hdrvals(2) = gy_check
       hdrvals(3) = hor_slow
       hwc = sep_put_val_headers ("pcorr",num_rec,1,hdrvals)
       hwc = sep_put_val_headers ("zcorr",num_rec,1,hdrvals)
       hwc = sep_put_val_headers ("pzcross",num_rec,1,hdrvals)
  deallocate(picking, component_traces, corr_buffer)
  call sep_close()
  call exit (0)
end program Corr
