!!$=head1 NAME
!!$
!!$Desig_north_sea - Signature Deconvolution for the North Sea OBN data
!!$
!!$=head1 SYNOPSIS
!!$
!!$Desig_north_sea < in.H signature_h= signature_g= compkey="laga" epsi=1e-8 > out.H
!!$
!!$=head1 DESCRIPTION
!!$
!!$Applies signature deconvolution in the Fourier domain. It takes the hydrophone response and the geophone response
!!$ and deconvolve the data with the corresponding signature.
!!$
!!$=head1 PARAMETERS
!!$
!!$=over 3
!!$
!!$=item from history 
!!$
!!$=item from par
!!$
!!$B<filename> I<signature_h> - signature file of the hydrophone
!!$
!!$B<filename> I<signature_g> - signature file of the geophone
!!$
!!$B<string> I<compkey> - header key that tells the kind of the component recorded
!!$
!!$B<real> I<epsi> - damping factor
!!$
!!$=back
!!$
!!$=head1 EXAMPLES
!!$
!!$=head1 MODULES
!!$
!!$=head1 SEE ALSO
!!$
!!$=cut
!%
program Desig
  use sep
  use fourier
  use resample
  integer                               :: nl, stat, i, rc, mh, k, sreed, ftlen, mpadh, mpadg, nlpad, mg
  integer :: iheader, compix, headlen
  integer, dimension(:),allocatable :: header
  real                                  :: sign1, d1filth, d1filtg, d1data
  real				:: epsi, epsg, epsh, epsd
  complex, dimension (:,:), allocatable :: caah, caag, cpp, cqq
  real,    dimension (:,:), allocatable   :: cpp_amp ! input data amplitude
  real,    dimension (:,:), allocatable   :: caah_amp ! hydrophone signature amplitude
  real,    dimension (:,:), allocatable   :: caag_amp ! geophone signature amplitude
  real,    dimension (:), allocatable   :: pp, qq!pp=input trace qq=out trace
  real,    dimension (:), allocatable   :: aah, aag !signature coefficients
  character*16 :: compkey
  common / deopt / rc
  call sep_init (SOURCE)
  call init_3d ()
  call from_either ("n1", nl)
  call from_aux ("signature_h","n1", mh)
  call from_aux ("signature_g","n1", mg)
  call from_par ("epsi",epsi,1.e-8)
  call from_par ("compkey",compkey,'laga')
  call from_either ("d1",d1data)
  call from_aux ("signature_h","d1", d1filth)
  call from_aux ("signature_g","d1", d1filtg)


  call sep_get_key_index("in", compkey, compix) 
  call sep_get_number_keys("in", headlen)
  allocate(header(headlen))
  iheader=1  
!Inizialize all the variables
  sign1 = 1.
  allocate (pp (nl), qq (nl))
  allocate (aah (mh), aag (mg) )
  pp = 0.
  aah = 0.
  aag = 0.
  qq = 0.

  call sep_read (aah, "signature_h") !read the filter coefficients for hydrophone
  call sep_read (aag, "signature_g") !read the filter coefficients for geophone
  
  call filt_resamp(d1filth,d1data,aah,mh, mpadh)
  call filt_resamp(d1filtg,d1data,aag,mg, mpadg)



  ftlen = max(mpadh,mpadg,nl)
  allocate (cpp (ftlen,1), cqq (ftlen,1), caah (ftlen,1), caag (ftlen,1))
  allocate (cpp_amp (ftlen,1), caah_amp (ftlen,1), caag_amp (ftlen,1))

  caag = 0.
  caah = 0.
  caag_amp = 0.
  caah_amp = 0.

  call ft_init(ftlen,1)

!shift the negative t-axis part to the positive axis
  aag = cshift(aag, shift = mpadg/4)
  caag(1:mpadg/2,1) = cmplx(aag(1:mpadg/2), 0.)
  caag((ftlen-mpadg/2):ftlen,1) = cmplx(aag((mpadg/2+1):mpadg), 0.)	

  aah = cshift(aah, shift = mpadh/4)
  caah(1:mpadh/2,1) = cmplx(aah(1:mpadh/2), 0.)
  caah((ftlen-mpadh/2):ftlen,1) = cmplx(aah((mpadh/2+1):mpadh), 0.)	

  !FFT  
  call ft1axis(.false., sign1, caag)
  call ft1axis(.false., sign1, caah)

  epsg     = epsi*(sum(cabs(caag)))/ftlen
  caag_amp = sqrt (caag * conjg(caag))
  caag     = caag / (caag_amp + epsg)

  epsh     = epsi*(sum(cabs(caah)))/ftlen
  caah_amp = sqrt (caah * conjg(caah))
  caah     = caah / (caah_amp + epsh)

! Reading the data
  rc=1
  do while (rc > 0)
    rc=sreed("in" , pp , nl*4)
    !write (0,*) 'Bytes read =',rc
   if (rc < 1) exit
    call sep_get_val_headers("in", iheader, 1, header)
    iheader= iheader+1
    cpp = 0.
    cpp_amp = 0.
    cqq = 0.
    cpp(1:nl,1) = pp(:)
    !FFT 
    call ft1axis(.false., sign1, cpp)   

	epsd    = epsi*(sum(cabs(caag)))/ftlen
	cpp_amp = sqrt (cpp * conjg(cpp))
	cpp     = cpp / (cpp_amp + epsd)

    !Deconvolve the data 
	if (header(compix) == 1) then
		cqq = conjg(caah) * cpp * (1./sqrt(1.*ftlen)) * (cpp_amp / (caah_amp + epsd))
	else
		cqq = conjg(caag) * cpp * (1./sqrt(1.*ftlen)) * (cpp_amp / (caag_amp + epsd))
	endif

    !IFFT
    call ft1axis(.true., sign1, cqq)
    qq(:) = real(cqq(:,1))
    if (mod(iheader,10000) == 0) then
       write (0,*) 'Processed traces:',iheader
    endif
    call sep_write (qq)
  enddo

  deallocate (pp, qq, aah, aag, header)
  deallocate (caag,caah, cpp, cqq)
  deallocate (caag_amp,caah_amp, cpp_amp)
  call ft_destroy()
  call sep_close()
  call exit (0)
end program Desig
