!!$=head1 NAME
!!$
!!$Filter_PZ - Program that computes the Wiener filter needed for the PZ summation
!!$
!!$=head1 SYNOPSIS
!!$
!!$Filter_PZ  pcorr= zcorr= pzcross= filter_length= epsi=0.1 > filter.H
!!$
!!$=head1 DESCRIPTION
!!$
!!$ Compute the Wiener shaping filter for the PZ summation considering the z axes pointing upward
!!$ so that U(f) = 1/2*P(f) + filt(f)*rho/(2*q)*Z(f)
!!$
!!$=head1 PARAMETERS
!!$
!!$=over 3
!!$
!!$=item from history 
!!$
!!$=item from par
!!$
!!$B<filename> I<pcorr> - Name of the file containing the Autocorrelation of the Pressure component
!!$
!!$B<filename> I<zcorr> - Name of the file containing the Autocorrelation of the vertical component
!!$
!!$B<filename> I<pzcross> - Name of the file containing the Crosscorrelation of the Pressure with the vertical component
!!$
!!$B<real> I<filter_length> - Length of the Shaping filter in seconds
!!$
!!$B<real> I<epsi> - Percentage of the white noise for the prewithening (default value 0.1%)
!!$
!!$
!!$=back
!!$
!!$=head1 EXAMPLES
!!$
!!$=head1 MODULES
!!$
!!$=head1 SEE ALSO
!!$
!!$=cut
!%
program Filter_PZ

  use sep
  use Wiener_filters

  implicit none
  integer                                :: n1, n2
  real                                   :: d1, epsi
  character*256  		    	 :: pcorrfile, zcorrfile, pzcrossfile
  real    				 :: filter_length !shaping filter length
  integer				 :: n_filter !number of samples of the filter
  real, allocatable, dimension(:,:)	 :: pauto, zauto, pzcross
  real, allocatable, dimension(:,:)	 :: filter
  integer				 :: zero_ix, i
  integer, dimension(3)			 :: header
  !open(unit=30,file="coor.out")




! Call initializer for SEP routines
  call sep_init (SOURCE)
  call init_3d ()
! Retrieve the information from parameter and aux files 
  call from_par ("filter_length",filter_length)
  !call from_par ("pcorr",pcorrfile)
  call from_par ("zcorr",zcorrfile)
  call from_par ("pzcross",pzcrossfile)
  call from_par ("epsi",epsi,0.1)

  call from_aux("zcorr","n1",n1)
  call from_aux("zcorr","d1",d1)
  call from_aux("zcorr","n2",n2)
  

  !Initialize internal variables
  epsi = epsi / 100.
  n_filter = nint(filter_length/d1)+1 
  allocate(zauto(n1,n2),pzcross(n1,n2))
  allocate(filter(n_filter,n2))
  filter(:,:) = 0.



  !Reading the data
  !call sep_read(pauto,"pcorr")
  call sep_read(zauto,"zcorr")
  call sep_read(pzcross,"pzcross")


  
! Write header information to the output filter
  call to_history("n1",n_filter)
  call to_history("o1",0.)
  call to_history("d1",d1)
  call to_history("n2",n2)
  call to_history("d2",1.)
  call to_history("label1","Time s")
  !Call the shaping deconvolution
  zero_ix = (n1-1)/2 + 1
  !Reverse order of pzbuff because we need zpcross and take the negative part
  do i = 1, n2
     !call sep_get_val_headers("zcorr", i, 1, header)
     !write(30,*) i, header(1),header(2), 1. /header(3)
     !zauto(zero_ix:1:-1,i) = (zauto(zero_ix:1:-1,i) + zauto(zero_ix:n1,i) ) / 2.
     call Shaping(zauto(zero_ix:1:-1,i),pzcross(zero_ix:1:-1,i),filter(:,i),epsi)
  enddo
  !Write the filter coefficients
  call sep_write(filter)

  !deallocate(pauto,zauto,pzcross)
  deallocate(zauto,pzcross,filter)
  !close(30)
  call sep_close()
  call exit (0)
end program Filter_PZ
