!!$=head1 NAME
!!$
!!$Pzsum - Program that perfoms the PZ summation
!!$
!!$=head1 SYNOPSIS
!!$
!!$Pzsum < in.H filter= water_vel=1500. upgoing= downgoing= > /dev/null
!!$
!!$=head1 DESCRIPTION
!!$
!!$ Perfom the wavefield separation considering a costant velocity model using the water velocity
!!$ given by the user in the command line. The shaping filter (i.e. calibration filter)
!!$ Note that the filter and the input file must have the same temporal sampling and
!!$ it is necessary to give the program all the components (P,x,y,z)
!!$
!!$=head1 PARAMETERS
!!$
!!$=over 3
!!$
!!$=item from history
!!$
!!$=item from par
!!$
!!$B<filename> I<filter> - Name of the file containing the calibration filter for each receiver.
!!$
!!$B<real> I<water_vel> - Water velocity (default value 1500 [m/s])
!!$
!!$B<filename> I<upgoing> - Name of the output file containing upgoing wavefield
!!$
!!$B<filename> I<downgoing> - Name of the output file containing downgoing wavefield
!!$
!!$=back
!!$
!!$=head1 EXAMPLES
!!$
!!$=head1 MODULES
!!$
!!$=head1 SEE ALSO
!!$
!!$=cut
!%

program Pzsum

  use sep
  use fourier
  use slope

  implicit none
  integer :: n1, n2, n1f, n2f
  real :: d1, o1, wat_vel, sign1, vert_slow_refrac, offset, tmax
  real, allocatable, dimension(:,:) :: filter
  integer, dimension(:), allocatable :: header, header_f
  integer :: PCOMP, XCOMP, YCOMP, ZCOMP
  character*256 :: filterfile
  integer :: GXHD, GYHD, VELHD ! Index for the header of the filter (it has only three components)
  integer :: gxid, gyid, sxid, syid, lagaid, gwdepid, headlen, iheader
  integer :: ptrace_ix, ztrace_ix, itrace
  integer :: sreed, srite
  integer :: rc, wc
  real, dimension(:,:),allocatable :: component_traces
  integer :: rec_id, rec_prev, maxlen
  complex, dimension(:,:), allocatable :: ctrace, cfilter
  real, dimension(:),allocatable :: upgoing, downgoing, slowness
  integer :: hc, sep_copy_header_keys,sep_put_header_axis_par, sep_put_val_headers, sep_put_number_keys, sep_copy_hff
  integer :: ih, sep_put_key, sep_get_key_name, sep_get_key_type, sep_get_key_fmt
  character*120 :: keyname, keytype, keyformat


  parameter(PCOMP=1,XCOMP=3,YCOMP=4,ZCOMP=2)
  parameter(GXHD=1,GYHD=2,VELHD=3)


  call sep_init ("/data_temp/north_sea_obn/desig/Src/Pzsum.f90")
  call init_3d ()

  call from_par ("filter",filterfile)
 !read the data information
  call from_history("n1",n1)
  call from_history("d1",d1) !Same temporal sampling of the filter
  call from_history("o1",o1)
  call from_history("n2",n2)
 !read filter information
  call from_aux("filter","n1",n1f)
  call from_aux("filter","n2",n2f)
  call from_par ("water_vel",wat_vel,1500.)
  !Allocate memory for the calibration filter and reading it from the parameter file
  allocate(filter(n1f,n2f))
  call sep_read(filter,"filter")
 !Header information of the input file
  call sep_get_key_index("in", 'gx', gxid)
  call sep_get_key_index("in", 'gy', gyid)
  call sep_get_key_index("in", 'sx', sxid)
  call sep_get_key_index("in", 'sy', syid)
  call sep_get_key_index("in", 'laga', lagaid)
  call sep_get_key_index("in", 'gwdep', gwdepid)
  call sep_get_number_keys("in", headlen)


 !Allocate variables
  allocate(header(headlen))
  allocate(component_traces(n1,4))
  allocate(header_f(3))
  maxlen = max(n1, n1f)
  allocate(ctrace(maxlen,1), cfilter(maxlen,1))
  allocate(upgoing(maxlen), downgoing(maxlen), slowness(maxlen))
 !Allocate space for the FFT
  call ft_init(maxlen,1)
 !Initialize the variables
  sign1 = 1.
  ctrace = 0.
  cfilter = 0.
  rec_prev = 0
  slowness = 0.
  tmax = o1 + (maxlen - 1)*d1


 !Put the dimension of the output files
  call to_history("n1",maxlen,"upgoing")
  call to_history("d1",d1,"upgoing")
  call to_history("o1",o1,"upgoing")
  call to_history("n2",n2/4,"upgoing")
  call to_history("d2",1.,"upgoing")
  call to_history("o2",1,"upgoing")
  call to_history("n1",maxlen,"downgoing")
  call to_history("d1",d1,"downgoing")
  call to_history("o1",o1,"downgoing")
  call to_history("n2",n2/4,"downgoing")
  call to_history("d2",1.,"downgoing")
  call to_history("o2",1,"downgoing")



 !Copy the header keys from input to output
  !hc = sep_copy_header_keys("in", "upgoing")
  !hc = sep_copy_header_keys("in", "downgoing")
  !hc = sep_copy_hff("in", "upgoing")
  !hc = sep_copy_hff("in", "downgoing")
  hc = sep_put_header_axis_par("upgoing",2,n2/4,1.,1.,"trace number")
  hc = sep_put_header_axis_par("downgoing",2,n2/4,1.,1.,"trace number")
  hc = sep_put_number_keys("upgoing",headlen)
  hc = sep_put_number_keys("downgoing",headlen)
  do ih = 1, headlen
      hc = sep_get_key_name("in",ih, keyname)
      hc = sep_get_key_type("in",ih, keytype)
      hc = sep_get_key_fmt("in",ih, keyformat)
      hc = sep_put_key("upgoing", keyname, keytype, keyformat,ih)
      hc = sep_put_key("downgoing", keyname, keytype, keyformat,ih)
  enddo





  iheader=1
  rc=1
  do while (rc > 0)
    !read for traces at each time
    rc=sreed("in" , component_traces(:,1) , n1*4)
    rc=sreed("in" , component_traces(:,2) , n1*4)
    rc=sreed("in" , component_traces(:,3) , n1*4)
    rc=sreed("in" , component_traces(:,4) , n1*4)
    if(rc < 1) exit!last_gather=.true.
    do itrace= 1,4
       call sep_get_val_headers("in", iheader, 1, header)
       iheader = iheader+1
       if (header(lagaid) == PCOMP) ptrace_ix=itrace
       if (header(lagaid) == ZCOMP) ztrace_ix=itrace
    enddo

    !Find to which receiver gather the trace belongs
    do rec_id = 1, n2f
       call sep_get_val_headers("filter", rec_id, 1, header_f)
       if ( header(gxid).eq.header_f(GXHD) .and. header(gyid).eq.header_f(GYHD)) exit
    enddo
    if (rec_id .ne. rec_prev) then
       vert_slow_refrac = sqrt(wat_vel**-2. - real(header_f(VELHD)**-2.)) !vertical slowness of the refracted arrivals
       cfilter = 0.
       cfilter(1:n1f,1) = filter(:,rec_id)
       call ft1axis(.false., sign1, cfilter)
       if (rec_prev .ne. 0)write(0,*)"Receiver processed:", rec_prev
       rec_prev = rec_id
    endif
    ctrace(1:n1,1) = component_traces(:,ztrace_ix)

    !Trasform the Z-component trace and convolve with the filter
    call ft1axis(.false., sign1, ctrace)
    ctrace(:,1) = cfilter(:,1) * ctrace(:,1) * sqrt(maxlen * 1.) !Get rid of the normalization factor in excess
    call ft1axis(.true., sign1, ctrace)

    !Computation of the absoffset and vertical slowness
    offset = abs(sqrt(real(header(gxid)-header(sxid))**2.+real(header(gyid)-header(syid))**2.))/ 100. !Conversion from cm to m
    call slow(slowness,vert_slow_refrac,offset,wat_vel,tmax,d1,maxlen,header(gwdepid))
    !Perfom the PZ summation for the wavefield separation
    upgoing = (component_traces(:,ptrace_ix) + slowness(:)*real(ctrace(:,1)))/2.
    downgoing = (component_traces(:,ptrace_ix) - slowness(:)*real(ctrace(:,1)))/2.
    !upgoing = component_traces(:,ptrace_ix)/2.
    !downgoing = slowness(:)*real(ctrace(:,1))/2.
    !Write the output files and the header of corresponding trace
    wc=srite("upgoing",upgoing,n1*4)
    wc=srite("downgoing",downgoing,n1*4)
    hc = sep_put_val_headers ("upgoing",(iheader-1)/4,1,header)
    hc = sep_put_val_headers ("downgoing",(iheader-1)/4,1,header)
    ctrace = 0.
    slowness = 0.
  enddo

  deallocate(upgoing,downgoing,slowness)
  deallocate(ctrace,cfilter)
  deallocate(filter,header,component_traces,header_f)
  call sep_close()
  call exit (0)
end program Pzsum
