!<
!read_geometry
!______________________________________________________________________
! read in some data and extract some headers using SEPlib3d
!______________________________________________________________________
! USAGE:
! read_geometry < in.H > out.H
!%end of self-documentation
!______________________________________________________________________
program hmo3d

use myhmo_mod
!______________________________________________________________________
! access the sep module
!______________________________________________________________________
IMPLICIT NONE

!______________________________________________________________________
! allocatable arrays
!______________________________________________________________________
REAL,		ALLOCATABLE, DIMENSION(:,:) :: data		! input seismic data
REAL,		ALLOCATABLE, DIMENSION(:,:) :: data_hmo		! output seismic data
INTEGER,		ALLOCATABLE, DIMENSION(:,:) :: headers	! HEADER BLOCK
REAL,		ALLOCATABLE, DIMENSION(:) :: offsetx	! source-receiver offset
REAL,		ALLOCATABLE, DIMENSION(:) :: offsety	! source-receiver offset
REAL,		ALLOCATABLE, DIMENSION(:) :: gelev	! source-receiver offset
integer,	ALLOCATABLE, DIMENSION(:) :: drn		! data record number
!______________________________________________________________________
! LOCATION OF VARIOUS KEYS IN THE HEADER STRUCTURES
!______________________________________________________________________
integer ::offsetx_key, offsety_key, gelev_key, drn_key
!______________________________________________________________________
! simple variables
!______________________________________________________________________
integer :: nt				! number of input samples
real	:: ot, dt
integer :: nkeys			! number of keys
integer :: ntrace			! total number of input traces
integer :: nh				! number of headers per trace
integer :: maxsize			! memory available for storage in MB of traces.
integer :: maxtr_per_block	! maximum number of traces in a block of data.
integer :: ntr_per_block	! number of traces in current block of data.
integer :: nblocks			! number of blocks of needed to read all the traces.
integer :: jblock			! block index
integer :: fwind			! first trace of the data window to be read.
integer :: ierr				! return value checker
logical :: have_drn			! WHETHER OR NOT DRN EXISTS
real	:: sdepth			! source depth
real	:: scaleco			! scaling for receiver elevation units
real	:: vel				! constant velocity for NMO in km/s
real	:: eps = 1e-4
logical :: verb
logical	:: first_hmo = .true. 
integer	:: ntraces_read = 0		! number of traces that will be written to output
!______________________________________________________________________
! control variables
!______________________________________________________________________
integer :: allocation_status ! return code for Fortran90 allocate command
integer :: stderr=0 ! standard error unit
!______________________________________________________________________
! DUMMY VARIALBES
!______________________________________________________________________
real :: o,dd
integer :: i, itrc ! LOOP VARIABLE
character(len=1025) :: label

!EXTERNAL SHOULD BE USED FOR ALL C CALLS, SOME F90 COMPILERS GET MAD WITHOUT IT
integer,external :: getch				! SEPlib get param from parameter file
integer,external :: hetch				! SEPlib get param from history file
integer,external :: fetch				! SEPlib get from either parameter or history file
integer,external :: putch				! SEPlib put to history file
integer,external :: sep_get_number_keys	! SEPlib number of keys
integer,external :: sep_get_key_index	! SEPlib index of given header key
integer,external :: sseek_block			! SEPlib SEEK FILE ROUTINE
integer,external :: sreed				! SEPlib READ ROUTINE
integer,external :: srite				! SEPlib WRITE ROUTINE
integer,external :: sep_get_header_axis_par		! SEPlib HEADER AXIS ROUTINE
integer,external :: sep_get_number_header_axes	! SEPlib HEADER AXIS ROUTINE
!_______________________________________________________________
! initialize the input seismic data file.
! do i need to specify any arguments?
!_______________________________________________________________
!TO INITIALIZE SEP IO/ PARA HANDLING
call initpar()
!SO SELF DOC WORKS
call doc(SOURCE)
!TO INITIALiZE SEP3D IO
call init_3d()
!_______________________________________________________________
! read in important parameters from the history or parameter file
!_______________________________________________________________
if (hetch('n1','d',nt ) <= 0 ) call seperr('cant read n1 from history file!')
if (hetch('o1','f',ot ) <= 0 ) call seperr('cant read o1 from history file!')
if (hetch('d1','f',dt ) <= 0 ) call seperr('cant read d1 from history file!')
if (getch('maxsize','d',maxsize ) <= 0 ) maxsize=1000000
if (getch('sdepth','f',sdepth) <= 0 ) sdepth=0. ! source depth in kilometers, some kind of average of shot depths to make things eaasier
if (getch('scaleco','f',scaleco) <= 0 ) scaleco=1. ! scale receiver elevation units by this to get to kilometers
if (getch('vel','f',vel) <= 0 ) vel=1.5 ! constant velocity for NMO in km/s, default=water
if (getch('verb','d',verb) <= 0 ) verb=.false.

if (verb) write (stderr,*) 'maxsize=', maxsize, 'sdepth=', sdepth

!THE NUMBER OF TRACES IS IN THE HEADER FILE
!IT DOESN'T HAVE TO BE THE SAME IS IN THE HISTORY FILE
!THEREFORE
if(sep_get_number_header_axes("in",ierr) .ne. 0) call seperr("trouble obtaining the number of header axes")

if(ierr .ne. 2) call seperr("for right now only works with 2-D header file")

! read from axis 2: number of traces
if(sep_get_header_axis_par("in", 2, ntrace, o, dd, label) .ne. 0) call seperr("trouble obtaining the number of headers")
!_______________________________________________________________
! read in important parameters from the hff file
!_______________________________________________________________
if (sep_get_number_keys("in",nkeys) .ne. 0) call seperr("trouble obtaining number of keys from hff")

if (sep_get_key_index("in", "offset", offsetx_key) .ne. 0) call seperr("trouble obtaining Offsetx key number")
if (sep_get_key_index("in", "offset", offsety_key) .ne. 0) call seperr("trouble obtaining Offsety key number")
if (sep_get_key_index("in", "gelev",   gelev_key)  .ne. 0) call seperr("trouble obtaining Gelev key number")

ierr = sep_get_key_index("in","data_record_number",drn_key)

if (ierr .ne. 0) then !DRN DOESN'T EXIST;

	if (getch ("same_record_number","d",ierr) == 1) then !IF SAME RECORD NUMBER IN HISTORY
		if(ierr==0) call seperr("no data_record_number but traces out of order with data")
	end if
	have_drn=.false.
else
	have_drn=.true.
end if

!______________________________________________________________________
! determine how many traces can fit into maxsize words of memory.
! determine how many blocks will need to be read in to read through
! all the input data.
!______________________________________________________________________
maxtr_per_block=(maxsize*1024.*1024.-1.)/(ntrace*nt)+1
nblocks=(ntrace-1)/maxtr_per_block+1

!______________________________________________________________________
! ALLOCATE arrays
! note that output arrays are of length nt (the same as the input).
! arrays used in Fourier transform are of length nt_fft >= nt.
!______________________________________________________________________
allocate (data (0:nt-1,maxtr_per_block), data_hmo (0:nt-1,maxtr_per_block), headers (nkeys,maxtr_per_block), &
	gelev (maxtr_per_block), offsetx (maxtr_per_block), offsety (maxtr_per_block), drn (maxtr_per_block), &
	STAT=allocation_status)

if(allocation_status /= 0) THEN
	write (stderr,*) 'space for data,headers,gelev,offsetx could not be allocated!'
	write (stderr,*) 'program aborted!'
	stop 9666
endif
!_______________________________________________________________________
! write out the history files defining the size of the output.
!_______________________________________________________________________
!LETS DEAL WITH THE OUTPUT
!WE ARE GOING TO REORDER THE TRACES, SO LETS TELL THE PROGRAM THAT
!THE TRACES ARE IN THE SAME ORDER

if(0 .ne. putch ("same_record_number","d",1)) call seperr("trouble putching same_record_number")

!IN THIS PROGRAM THE OUTPUT IS THE SAME SIZE OF
!THE INPUT. THE HISTORY FILE BY DEFAULT IS THE SAME
!SIZE AS THE INPUT SO WE DON'T NEED TO DO ANY MANIPULATION
!
!THE HFF FILE DOES NEED TO BE CREATED. SAYING WE
!HAVE THE SAME KEYS. WE CAN JUST COPY THE HFF FILE
call sep_copy_hff("in","out")



!___________________________________________________________________
! loop over the blocks of data.
!___________________________________________________________________
loop_over_blocks: do jblock=1,nblocks

	!___________________________________________________________________
	! calculate the number of traces in this block.
	!___________________________________________________________________
	ntr_per_block=maxtr_per_block
	if (jblock == nblocks) ntr_per_block = ntrace-(nblocks-1)*maxtr_per_block
	!___________________________________________________________________
	! grab the next ntr_per_block trace headers from the trace header file.
	! 'in' = standard in?
	! headers = a fortran90 structure
	! nh = the number of headers in the structure.
	! fwind = the sequential trace position in the trace header file.
	! or is the area where the next position on the file will be
	! mapped?
	!___________________________________________________________________
	fwind=(jblock-1)*maxtr_per_block

	!READ IN THE HEADERS IN THIS BLOCK
	call sep_get_val_headers("in", fwind+1, ntr_per_block, headers)
	!_______________________________________________________________________
	! echo out how many headers are out there on this file.
	!_______________________________________________________________________
	!___________________________________________________________________
	! grab a subset of desired headers using the header key words.
	!___________________________________________________________________
	offsetx = headers (offsetx_key, :)
	offsety = headers (offsety_key, :)
	gelev   = abs (headers (gelev_key,   :) * scaleco - sdepth) ! scale receiver depth and subtract source depth

	offsetx = offsetx/1000.!sqrt (offsetx * offsetx + offsety * offsety)

	!IF WE HAVE A DATA RECORD NUMBER WE NEED TO GRAB THE
	!CORRESPONDING TRACE NUMBERS. THE HEADER BLOCK IS
	!REAL, DRN ARE INTS. WE THEREFORE NEED TO CONVERT

	ntraces_read = ntraces_read + ntr_per_block

	if(have_drn) then
		drn(1:ntr_per_block) = transfer (headers (drn_key, :ntr_per_block), 0, ntr_per_block)
	
		do i=1,ntr_per_block
			!___________________________________________________________________
			! now let's try to read ntr_per_block traces of seismic data.
			!___________________________________________________________________
			!SEEK TO THE BEGINING OF THE GIVEN TRACE
			if (sseek_block ("in", drn(i)-1, 4*nt, 0) < 0) call seperr("trouble seeking trace")
			if (sreed ("in", data (:,i),   4*nt) .ne. 4*nt)    call seperr("trouble reading trace ")
			drn(i)=i+fwind !RESET DRN TO ITS NEW POSITION
		end do
	else !TRACES ARE IN ORDER SO JUST READ THEN
	!___________________________________________________________________
	! now let's try to read ntr_per_block traces of seismic data.
	!___________________________________________________________________
		if (sreed("in", data, 4*nt*ntr_per_block) .ne. 4*nt*ntr_per_block) call seperr("trouble reading trace block")
	end if

	data_hmo = 0.

	!___________________________________________________________________
	! do hyperbolic moveout
	!___________________________________________________________________

	call hmo_init (1./vel, gelev/vel, offsetx, nt, ot, dt, .false., first_hmo)
	call hmo_nearest (data, data_hmo)
	
	!LETS ACTUALLY WRITE OUT OUR CHANGES TO THE OUTPUT
	
	!WRITE THE DATA
	if (srite ("out", data_hmo, 4*nt*ntr_per_block) .ne. 4*nt*ntr_per_block) call seperr("trouble writing out data")

	!PUT THE HEADERS BACK OUT
	call sep_put_val_headers ("out", fwind+1, ntr_per_block, headers)

end do loop_over_blocks

! write out the number of traces we actually dealt with.
if(0 .ne. putch ("n2","d",ntraces_read)) call seperr("trouble putching ntraces_read")

!_____________________________________________________________
! close sep data files
!_____________________________________________________________
call hclose()

if (verb) write (stderr,*) 'hmo3d: normal completion'

end program hmo3d

