!Module to read the picking table for PZ and create a related object
! Pick file must be sorted by common receiver location (gx,gy), common source position (sx,sy), picked time (t)
module pick_obj
 use ieee_arithmetic
 implicit none
 type pick_entry
  real (kind=8), dimension(2) :: t
  real (kind=8) :: gx,gy
  real (kind=8), dimension(2) :: sx,sy
 end type pick_entry
 type pick
  type(pick_entry), pointer, private :: pick_list(:)
  real (kind=8), dimension(:,:), allocatable, private :: unit_vector ! component of the unit vector of the picked receivers
  real (kind=8)     :: gxchk, gychk
  real(kind=8)	    :: tntrp(2), sxntrp(2), syntrp(2), usx, usy
  contains
    procedure, pass(pick_arg) :: decode_pick_file
    procedure, pass(pick_arg) :: deallocate_pick
    procedure, pass(pick_arg) :: writing_values
    procedure, pass(pick_arg) :: window
    procedure, pass(pick_arg) :: vector
 end type

contains

 subroutine decode_pick_file (pick_arg, filename)
    implicit none
    character*(*), intent(in) :: filename
    class(pick), intent(inout) :: pick_arg

    character*256 :: line_buffer
    integer :: open_stat,read_stat
    integer :: line_count, rec_count
    logical :: first
    type(pick_entry) :: pick, previous_pick
    !Open the file to be read and count the number of receivers
    open(file=filename,unit=22,form='formatted', status='old', iostat=open_stat)
    if (open_stat .ne. 0 ) then
       write(0,*) "Cannot read the input file"
       return
    endif
    
    read_stat=0
    line_count=0
    rec_count=0
    first=.true.
    do while (read_stat == 0)
     read(unit=22,fmt='(A)',iostat=read_stat) line_buffer
     if (read_stat == 0) then
        if (line_buffer(1:1)=='#') cycle
        line_count = line_count + 1
        read(line_buffer,*) pick%t(1), pick%sx(1), pick%sy(1), pick%gx, pick%gy
        if (first) then
            previous_pick=pick
            first=.false.
            rec_count= rec_count + 1            
        endif 
        if (pick%gx .ne. previous_pick%gx .or. pick%gy .ne. previous_pick%gy) then
           rec_count= rec_count + 1
        endif
        previous_pick=pick
     endif
    enddo


    rewind(unit=22)
    if (rec_count > 0) then
        allocate(pick_arg%pick_list(1:rec_count))
        read_stat=0
        first=.true.
        rec_count=0
        do while (read_stat == 0)
           read(unit=22,fmt='(A)',iostat=read_stat) line_buffer
           if (read_stat == 0) then
             if (line_buffer(1:1)=='#') cycle
             read(line_buffer,*) pick%t(1), pick%sx(1), pick%sy(1), pick%gx, pick%gy
             if (first) then
                previous_pick=pick
                first=.false.
                rec_count= rec_count + 1
                pick_arg%pick_list(rec_count)=pick
             endif 
             if (pick%gx .ne. previous_pick%gx .or. pick%gy .ne. previous_pick%gy) then
                 rec_count= rec_count + 1
                 pick_arg%pick_list(rec_count)=pick
             else
                 pick_arg%pick_list(rec_count)%sx(2)=pick%sx(1) 
                 pick_arg%pick_list(rec_count)%sy(2)=pick%sy(1)
                 pick_arg%pick_list(rec_count)%t(2)=pick%t(1)
             endif
             previous_pick=pick
           endif
        enddo
    endif

    close(unit=22)
    return
 end subroutine decode_pick_file
 subroutine deallocate_pick(pick_arg)
    class(pick), intent(inout) :: pick_arg
    deallocate(pick_arg%pick_list)
    return
 end subroutine deallocate_pick

 subroutine writing_values(pick_arg)
    implicit none
    class(pick), intent(in) :: pick_arg
    integer :: i
    do i = 1, size(pick_arg%pick_list)
       write(0,*) "Picking values rec pos. Gx = ", pick_arg%pick_list(i)%gx, " Gy = ", pick_arg%pick_list(i)%gy 
       write(0,*) "Time 1 = ", pick_arg%pick_list(i)%t(1), "Time 2 =", pick_arg%pick_list(i)%t(2)
       write(0,*) "Sx 1 = ", pick_arg%pick_list(i)%sx(1), "Sx 2 =", pick_arg%pick_list(i)%sx(2)
       write(0,*) "Sy 1 = ", pick_arg%pick_list(i)%sy(1), "Sy 2 =", pick_arg%pick_list(i)%sy(2)
    enddo
    write(0,*) ""
    return
 end subroutine writing_values
 
 subroutine vector(pick_arg)
   class(pick), intent(inout)   :: pick_arg
   real(kind=8)			:: dist, x, y
   integer   			:: pkid

   allocate(pick_arg%unit_vector(size(pick_arg%pick_list)-1,2))
   do pkid = 1, size(pick_arg%pick_list)-1
        x = pick_arg%pick_list(pkid+1)%gx - pick_arg%pick_list(pkid)%gx
        y = pick_arg%pick_list(pkid+1)%gy - pick_arg%pick_list(pkid)%gy
        dist = sqrt((x)**2. + (y)**2.)
        pick_arg%unit_vector(pkid,1)= x / dist
        pick_arg%unit_vector(pkid,2)= y / dist
   enddo
   return
 end subroutine vector

 subroutine vector_source(sx,sy,usx,usy)
   real(kind=8)			:: dist, x, y
   integer   			:: pkid
   real (kind=8), intent(in)	:: sx(2),sy(2)
   real (kind=8), intent(out)	:: usx,usy

   x = sx(2)- sx(1)
   y = sy(2)- sy(1)
   dist = sqrt((x)**2. + (y)**2.)
   usx = x / dist
   usy = y / dist
   return
 end subroutine vector_source

 subroutine lin_inter(inter,dot1,dot2,y1,y2)
    real(kind=8) :: inter,dot1,dot2,y1,y2,sumdot
    sumdot =  1. / (dot1 + dot2)
    inter = sumdot *(dot2 * y1 + dot1 * y2)
    return
 end subroutine lin_inter

 function window(pick_arg,gx,gy,sx,sy,t)
    implicit none
    !Input variables
    class(pick), intent(inout) :: pick_arg
    integer, intent(in)     :: gx, gy, sx, sy
    !Output variables
    logical		    :: window
    real(kind=8)	    :: t
    !Internal variables
    integer		    :: pkid
    real(kind=8)	    :: dot1, dot2

  

    !Checking in which picked receiver interval the input trace falls using unit vectors
    do pkid = 1, size(pick_arg%pick_list)-1
        !Calculate the dot product
        dot1=pick_arg%unit_vector(pkid,1)*(gx-pick_arg%pick_list(pkid)%gx)+ &
              pick_arg%unit_vector(pkid,2)*(gy-pick_arg%pick_list(pkid)%gy)
        dot2=-pick_arg%unit_vector(pkid,1)*(gx-pick_arg%pick_list(pkid+1)%gx)- &
              pick_arg%unit_vector(pkid,2)*(gy-pick_arg%pick_list(pkid+1)%gy)
        if(dot1 >= 0. .and. dot2 >= 0) then
            !The receiver is between two picked receivers 
  	    !Linear interpolation of time, sx, sy and computation of the source interpolated pick unit vectors
            if (pick_arg%gxchk .ne. gx .or. pick_arg%gychk .ne. gy) then
                call lin_inter(pick_arg%tntrp(1),dot1,dot2,pick_arg%pick_list(pkid)%t(1),pick_arg%pick_list(pkid+1)%t(1))
     		call lin_inter(pick_arg%tntrp(2),dot1,dot2,pick_arg%pick_list(pkid)%t(2),pick_arg%pick_list(pkid+1)%t(2))
                call lin_inter(pick_arg%sxntrp(1),dot1,dot2,pick_arg%pick_list(pkid)%sx(1),pick_arg%pick_list(pkid+1)%sx(1))
                call lin_inter(pick_arg%sxntrp(2),dot1,dot2,pick_arg%pick_list(pkid)%sx(2),pick_arg%pick_list(pkid+1)%sx(2))
                call lin_inter(pick_arg%syntrp(1),dot1,dot2,pick_arg%pick_list(pkid)%sy(1),pick_arg%pick_list(pkid+1)%sy(1))
                call lin_inter(pick_arg%syntrp(2),dot1,dot2,pick_arg%pick_list(pkid)%sy(2),pick_arg%pick_list(pkid+1)%sy(2))
                call vector_source(pick_arg%sxntrp,pick_arg%syntrp,pick_arg%usx,pick_arg%usy)
		pick_arg%gxchk = gx
		pick_arg%gychk = gy
            endif
            exit
        elseif (dot1 < 0. .and. pkid == 1) then
            !The receiver x coordinate is smaller of the left-end receiver
            if (pick_arg%gxchk .ne. gx .or. pick_arg%gychk .ne. gy) then
                call lin_inter(pick_arg%tntrp(1),dot1,dot2,pick_arg%pick_list(pkid)%t(1),pick_arg%pick_list(pkid+1)%t(1))
     		call lin_inter(pick_arg%tntrp(2),dot1,dot2,pick_arg%pick_list(pkid)%t(2),pick_arg%pick_list(pkid+1)%t(2))
                call lin_inter(pick_arg%sxntrp(1),dot1,dot2,pick_arg%pick_list(pkid)%sx(1),pick_arg%pick_list(pkid+1)%sx(1))
                call lin_inter(pick_arg%sxntrp(2),dot1,dot2,pick_arg%pick_list(pkid)%sx(2),pick_arg%pick_list(pkid+1)%sx(2))
                call lin_inter(pick_arg%syntrp(1),dot1,dot2,pick_arg%pick_list(pkid)%sy(1),pick_arg%pick_list(pkid+1)%sy(1))
                call lin_inter(pick_arg%syntrp(2),dot1,dot2,pick_arg%pick_list(pkid)%sy(2),pick_arg%pick_list(pkid+1)%sy(2))
                call vector_source(pick_arg%sxntrp,pick_arg%syntrp,pick_arg%usx,pick_arg%usy)
		pick_arg%gxchk = gx
		pick_arg%gychk = gy
            endif
            exit
        elseif (dot2 < 0. .and. (pkid+1) == size(pick_arg%pick_list)) then
            !The receiver x coordinate is greater than right-end receiver
            if (pick_arg%gxchk .ne. gx .or. pick_arg%gychk .ne. gy) then
                call lin_inter(pick_arg%tntrp(1),dot1,dot2,pick_arg%pick_list(pkid)%t(1),pick_arg%pick_list(pkid+1)%t(1))
     		call lin_inter(pick_arg%tntrp(2),dot1,dot2,pick_arg%pick_list(pkid)%t(2),pick_arg%pick_list(pkid+1)%t(2))
                call lin_inter(pick_arg%sxntrp(1),dot1,dot2,pick_arg%pick_list(pkid)%sx(1),pick_arg%pick_list(pkid+1)%sx(1))
                call lin_inter(pick_arg%sxntrp(2),dot1,dot2,pick_arg%pick_list(pkid)%sx(2),pick_arg%pick_list(pkid+1)%sx(2))
                call lin_inter(pick_arg%syntrp(1),dot1,dot2,pick_arg%pick_list(pkid)%sy(1),pick_arg%pick_list(pkid+1)%sy(1))
                call lin_inter(pick_arg%syntrp(2),dot1,dot2,pick_arg%pick_list(pkid)%sy(2),pick_arg%pick_list(pkid+1)%sy(2))
                call vector_source(pick_arg%sxntrp,pick_arg%syntrp,pick_arg%usx,pick_arg%usy)
		pick_arg%gxchk = gx
		pick_arg%gychk = gy
            endif
            exit
        endif
    enddo
    call pick_interp(pick_arg%tntrp,pick_arg%sxntrp,pick_arg%syntrp,pick_arg%usx,pick_arg%usy,sx,sy,t)  
    if (t == -huge(t)) then
       window = .false.
    else
       window = .true.
    endif
    return
 end function window
 
 subroutine pick_interp(tntrp,sxntrp,syntrp,usx,usy,sx,sy,t)
    !Linear intepolation 
    implicit none
    real(kind=8),intent(in) :: tntrp(2), sxntrp(2), syntrp(2), usx, usy
    real(kind=8)	    :: t
    integer, intent(in)     :: sx, sy
    real(kind=8)	    :: dot1, dot2, sumdot
    !Computed the dot products of the picked sources with the current source
    dot1 = usx * (sx - sxntrp(1)) + usy * (sy - syntrp(1))
    dot2 = -usx * (sx - sxntrp(2)) - usy * (sy - syntrp(2))
    if (dot1 >= 0. .and. dot2 >= 0) then
       !The source of the trace considered is inside the window
       !Interpolate to find the time for the considered trace
       sumdot = 1. / (dot1 + dot2)
       t = sumdot * ( tntrp(1) * dot2 + tntrp(2) * dot1 ) 
    else
       !The source considered is outside the window
       t=-huge(t)
    endif
    return
 end subroutine pick_interp
end module pick_obj




